
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CryptoRandom_cxx_Version =
    "$Id: CryptoRandom.cxx,v 1.10 2001/06/27 01:43:45 bko Exp $";




#include "CryptoRandom.hxx"
#include "support.hxx"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <errno.h>
#ifdef WIN32
#include <stdlib.h>
#endif

#include <unistd.h>
#include "cpLog.h"
#include "Data.hxx"


static bool srandomCalled = false;

CryptoRandom::CryptoRandom()
{
}



CryptoRandom::~CryptoRandom()
{

}





#ifdef __LINUX__

int CryptoRandom::getCryptoRandom(unsigned char* buf, int num)
{
    ssize_t len = 0;

    int fd;

    //open the io device /dev/random.
    fd = open( "/dev/random", O_RDONLY );

    if ( fd == -1 )
    {
        cpLog(LOG_DEBUG, "CryptoRandom::unable to open /dev/random");
    }
    else
    {

        len = read(fd, buf, num);

        buf[num] = '\0';

        if (len == -1)
        {
            cpLog(LOG_DEBUG, "CryptoRandom:: error no .  in reading: %d" , errno );
        }
        else
        {

            cpLog(LOG_DEBUG, " CryptoRandom::got random numbers");
        }
    }
    //close the io device /dev/random.
    int err = close(fd);
    if (err == -1)
    {
        cpLog(LOG_DEBUG, "Error occured when closing the /dev/random fd");
    }
    return len;
}

#endif


#ifndef WIN32
int CryptoRandom::getPseudoRandom(unsigned char* buf, int num)
{
    //generate a random number;
    struct timeval tv;
    gettimeofday(&tv, 0);

    if (!srandomCalled)
    {
        srandom((unsigned int)(tv.tv_usec ^ tv.tv_sec));
        srandomCalled = true;
    }

    int rand = random();

    rand += tv.tv_usec; ;



    memcpy(buf, &rand, 4);  //copy 4 bytes from rand to buf.

    Data hexval = convertToHex(buf, num);


    return num;
}
#else
int CryptoRandom::getPseudoRandom(unsigned char* buf, int num)
{
    //generate a random number;
    struct timeval tv;
    gettimeofday(&tv, 0);

    if (!srandomCalled)
    {
        srand((unsigned int)(tv.tv_usec ^ tv.tv_sec));
        srandomCalled = true;
    }
    int r = rand();
    r += tv.tv_usec; ;

    memcpy(buf, &r, 4);  //copy 4 bytes from rand to buf.
    //Data hexval = convertToHex(buf, num);
    return num;
}
#endif // WIN32

//char* CryptoRandom::getRandom()
int CryptoRandom::getRandom(unsigned char* buf, int num)
{


    int len;

#ifdef __LINUX__

#ifdef  USE_CRYPTO_RANDOM

    len = getCryptoRandom(buf, num);
#else

len = getPseudoRandom(buf, num);

    Data hexval = convertToHex(buf, num);

#endif


#else
//generate a random number;
    len = getPseudoRandom(buf, num);

#endif 
    return len;
}
