
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#ifdef WIN32

static const char* const Fifo_win32_cxx_Version =
    "$Id: Fifo-win32.cxx,v 1.2 2001/06/29 20:45:34 bko Exp $";


#include <cerrno>
#include "cpLog.h"
//#include "Fifo.h"

template < class Msg >
Fifo < Msg > ::Fifo()
{}



template < class Msg >
Fifo < Msg > ::~Fifo()
{}



template < class Msg >
void
Fifo < Msg > ::wakeup()
throw ( VException )
{
    // Tell any waiting threads the FIFO is no longer empty
    //
    int error = condition.signal();
    assert( !error );
}


template < class Msg >
int
Fifo < Msg > ::wait(milliseconds_t relativeTimeout)
throw ( VException )
{

    // This will unlock the object's mutex, wait for a condition's signal,
    // and relock.
    //
    int error = condition.wait(&mutex_, relativeTimeout);

    if ( error != 0 )
    {
        // If we have been awaken due to a unix signal, i.e. SIGTERM,
        // or a timeout return 0 to indicate no activity.
        //
        if ( error == EINTR || error == ETIMEDOUT )
        {
            return ( 0 );
        }

        // Defensive, shouldn't ever get here.
        //
        else
        {
            cpLog(LOG_ERR, "VCondition::wait: error = %s (%d)\n", strerror(error), error);
            assert( 0 );
        }
    }

    return ( fifo_.size() );
}

#endif // WIN32